import Api from "../src/api/Api";
import LunrIndex from "../src/api/LunrIndex";
import sampleConfig from './fixtures/sampleConfig';
import mockFetchReturn from '../__mocks__/mockFetchReturn';

import sampleDocumentsOne from './fixtures/documents-1';
import sampleDocumentsTwo from './fixtures/documents-2';
import sampleDocumentsThree from './fixtures/documents-3';

let lunr, index;

beforeEach((done) => {

  index = new Api(sampleConfig).getServer('site_content').getIndex('basic_pages');
  global.fetch = jest.fn();
  // Mock three consecutive calls to fetch with some sample documents.
  mockFetchReturn(global.fetch, sampleDocumentsOne);
  mockFetchReturn(global.fetch, sampleDocumentsTwo);
  mockFetchReturn(global.fetch, sampleDocumentsThree);

  lunr = new LunrIndex(index, {driver: 'memory'});

  lunr.cachedIndex.clear().then(() => {
    done();
  });
});

test('searching for a single document', (done) => {
  // Only one document has the word "Abbas".
  lunr.search('Abbas', 5).then((results) => {
    expect(results.length).toEqual(1);
    expect(results[0].getId()).toEqual('entity:node/264:en');
    done();
  });
});

test('searching for a multiple documents', (done) => {
  // Only one document has the word "Abbas".
  lunr.search('Vulpes', 5).then((results) => {
    expect(results.length).toEqual(2);
    done();
  });
});

test('searching for a multiple documents with a limit', (done) => {
  // Only one document has the word "Abbas".
  lunr.search('Vulpes', 1).then((results) => {
    expect(results.length).toEqual(1);
    done();
  });
});

test('test multiple searches hydrates index once', (done) => {
  // Only one document has the word "Abbas".
  let idx1, idx2;
  let searches = [
    lunr.hydrateIndex().then((idx) => {
      idx1 = idx;
    }),
    lunr.hydrateIndex().then((idx) => {
      idx2 = idx;
    }),
  ];
  Promise.all(searches).then(() => {
    expect(idx1 === idx2).toEqual(true);
    done();
  });
});

test('loading a cached index', (done) => {
  // Execute a search which will hydrate a cached index.
  lunr.search('Vulpes', 1)
    .then((results) => {
      expect(results.length).toEqual(1);
      expect(lunr._lastHydrateStrategy).toEqual('index');
    })
    .then(() => {
      // By creating a second lunr index, we simulate a page load, in the
      // browser context, the persistent cache would load the index, but in
      // the test environment a second instantiation will read from the
      // memory index.
      let secondLunrIndex = new LunrIndex(index, 'memory');
      secondLunrIndex.search('Vulpes', 1).then((results) => {
        expect(results.length).toEqual(1);
        expect(secondLunrIndex._lastHydrateStrategy).toEqual('cache');
        done();
      });
    });
});
